# Generated by Django 3.0.5 on 2020-04-08 02:09

from django.db import migrations, models
import django.db.models.deletion
import huscy.project_ethics.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('projects', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Ethic',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(blank=True, default='', max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='EthicBoard',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name='EthicFile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('filetype', models.PositiveSmallIntegerField(choices=[(0, 'Proposal'), (1, 'Votum'), (2, 'Amendment')])),
                ('filehandle', models.FileField(upload_to=huscy.project_ethics.models.EthicFile.get_upload_path)),
                ('filename', models.CharField(max_length=255)),
                ('uploaded_at', models.DateTimeField(auto_now_add=True)),
                ('uploaded_by', models.CharField(editable=False, max_length=126)),
                ('ethic', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='project_ethics.Ethic')),
            ],
        ),
        migrations.AddField(
            model_name='ethic',
            name='ethic_board',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='project_ethics.EthicBoard'),
        ),
        migrations.AddField(
            model_name='ethic',
            name='project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='projects.Project'),
        ),
    ]



from itertools import cycle

import pytest
from model_bakery import baker

from huscy.project_ethics.services import get_or_create_ethics

pytestmark = pytest.mark.django_db


def test_get_ethics_without_project():
    ethics = create_ethics()
    result = get_or_create_ethics()

    assert list(result) == ethics


def test_get_ethics_filtered_by_project():
    ethics = create_ethics()

    result = get_or_create_ethics(ethics[0].project)

    assert len(result) == 2
    assert list(result) == [ethics[0], ethics[3]]


def test_create_ethics_for_project(project):
    assert len(get_or_create_ethics()) == 0  # no ethics present

    result = get_or_create_ethics(project)  # create ethic

    assert len(get_or_create_ethics()) == 1

    assert list(result) == list(get_or_create_ethics(project))  # get but no create


def create_ethics():
    projects = baker.make('projects.Project', _quantity=3)
    return baker.make('project_ethics.Ethic', project=cycle(projects), _quantity=6)



from rest_framework.routers import DefaultRouter

from huscy.project_ethics import views


router = DefaultRouter()
router.register('ethicboards', views.EthicBoardViewSet)
router.register('ethicfiles', views.EthicFileViewSet)
router.register('ethics', views.EthicViewSet, basename='ethic')
router.register('projects', views.ProjectViewSet)



import pytest
from model_bakery import baker

from rest_framework.test import APIClient


@pytest.fixture
def user(django_user_model):
    return django_user_model.objects.create_user(username='user', password='password',
                                                 first_name='Erik', last_name='Zion')


@pytest.fixture
def admin_client(admin_user):
    client = APIClient()
    client.login(username=admin_user.username, password='password')
    return client


@pytest.fixture
def client(user):
    client = APIClient()
    client.login(username=user.username, password='password')
    return client


@pytest.fixture
def anonymous_client():
    return APIClient()


@pytest.fixture
def project():
    return baker.make('projects.Project')


@pytest.fixture
def ethic_board():
    return baker.make('project_ethics.EthicBoard')


@pytest.fixture
def ethic(ethic_board, project):
    return baker.make('project_ethics.Ethic', ethic_board=ethic_board, project=project)


@pytest.fixture
def ethic_file():
    return baker.make('project_ethics.EthicFile')


@pytest.fixture
def tmp_file(tmp_path):
    _file = tmp_path / 'tmp.txt'
    _file.write_text('foo bar')
    return _file



import pytest

from rest_framework.reverse import reverse
from rest_framework.status import HTTP_204_NO_CONTENT, HTTP_403_FORBIDDEN

pytestmark = pytest.mark.django_db


def test_admin_user_can_delete_ethic_files(admin_client, ethic_file):
    response = delete_ethic_file(admin_client, ethic_file)

    assert response.status_code == HTTP_204_NO_CONTENT


def test_user_without_permission_can_delete_ethic_files(client, ethic_file):
    response = delete_ethic_file(client, ethic_file)

    assert response.status_code == HTTP_204_NO_CONTENT


def test_anonymous_user_cannot_delete_ethic_files(anonymous_client, ethic_file):
    response = delete_ethic_file(anonymous_client, ethic_file)

    assert response.status_code == HTTP_403_FORBIDDEN


def delete_ethic_file(client, ethic_file):
    return client.delete(reverse('ethicfile-detail', kwargs=dict(pk=ethic_file.pk)))



from rest_framework import mixins, viewsets
from rest_framework.permissions import DjangoModelPermissions, IsAuthenticated

from huscy.project_ethics import models, serializer, services


class EthicViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixins.UpdateModelMixin,
                   viewsets.GenericViewSet):
    permission_classes = (IsAuthenticated, )
    queryset = services.get_or_create_ethics()
    serializer_class = serializer.EthicSerializer


class EthicBoardViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixins.ListModelMixin,
                        viewsets.GenericViewSet):
    permission_classes = (DjangoModelPermissions, )
    queryset = services.get_ethic_boards()
    serializer_class = serializer.EthicBoardSerializer


class EthicFileViewSet(mixins.CreateModelMixin, mixins.DestroyModelMixin, mixins.ListModelMixin,
                       viewsets.GenericViewSet):
    permission_classes = (IsAuthenticated, )
    queryset = services.get_ethic_files()
    serializer_class = serializer.EthicFileSerializer


class ProjectViewSet(mixins.RetrieveModelMixin, viewsets.GenericViewSet):
    permission_classes = (IsAuthenticated, )
    queryset = models.Project.objects.all()
    serializer_class = serializer.ProjectSerializer



from rest_framework import serializers

from huscy.project_ethics import models, services


class EthicBoardSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.EthicBoard
        fields = (
            'id',
            'name',
        )


class EthicSerializer(serializers.ModelSerializer):
    ethic_board_name = serializers.SerializerMethodField()
    ethic_files = serializers.SerializerMethodField()

    class Meta:
        model = models.Ethic
        fields = (
            'code',
            'ethic_board',
            'ethic_board_name',
            'ethic_files',
            'id',
            'project',
        )

    def get_ethic_board_name(self, ethic):
        return ethic.ethic_board and ethic.ethic_board.name

    def get_ethic_files(self, ethic):
        return EthicFileSerializer(ethic.ethicfile_set.all(), many=True).data


class EthicFileSerializer(serializers.ModelSerializer):
    filetype_name = serializers.SerializerMethodField()

    class Meta:
        model = models.EthicFile
        fields = (
            'ethic',
            'filehandle',
            'filename',
            'filetype',
            'filetype_name',
            'uploaded_at',
            'uploaded_by',
        )
        read_only_fields = 'filename',

    def get_filetype_name(self, ethic_file):
        return ethic_file.get_filetype_display()

    def create(self, validated_data):
        creator = self.context.get('request').user
        return services.create_ethic_file(**validated_data, creator=creator)


class ProjectSerializer(serializers.ModelSerializer):
    ethics = serializers.SerializerMethodField()

    class Meta:
        model = models.Project
        fields = (
            'id',
            'ethics',
        )

    def get_ethics(self, project):
        ethics = services.get_or_create_ethics(project)
        return EthicSerializer(ethics, many=True).data



import os


BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = ')zn(^y3s&=_b7r6+n-grwxnz(lu1mh@k+4tvopv@dawv%a-@1f'

DEBUG = True


INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',

    'django_filters',
    'rest_framework',

    'huscy.project_ethics',
    'huscy.projects',
]

# check if django extensions is installed because it's just an extras_require dependency
try:
    import django_extensions
    INSTALLED_APPS.append('django_extensions')
except ImportError:
    pass


MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

ROOT_URLCONF = 'config.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': 'huscy',
        'USER': 'huscy',
        'PASSWORD': '123',
        'HOST': 'localhost',
        'PORT': '5432',
    }
}


USE_L10N = True


STATIC_URL = '/static/'


MEDIA_ROOT = os.path.join(BASE_DIR, 'media/')
MEDIA_URL = '/media/'



from datetime import datetime

import pytest

from django.core.files import File

from huscy.project_ethics.models import EthicFile
from huscy.project_ethics.services import create_ethic_file


@pytest.mark.freeze_time('2020-01-01 10:00:00')
def test_create_ethic_file(user, ethic, tmp_file):
    filetype = EthicFile.TYPE.get_value('proposal')

    with open(tmp_file, 'r') as fh:
        filehandle = File(fh)
        result = create_ethic_file(ethic, filehandle, filetype, user)

        assert result.ethic == ethic
        assert result.filetype == filetype
        assert result.filename == 'tmp.txt'
        assert result.uploaded_at == datetime(2020, 1, 1, 10)
        assert result.uploaded_by == 'Erik Zion'



import pytest

from rest_framework.reverse import reverse
from rest_framework.status import HTTP_200_OK, HTTP_403_FORBIDDEN

pytestmark = pytest.mark.django_db


def test_admin_user_can_update_ethic(admin_client, ethic):
    response = update_ethic(admin_client, ethic)

    assert response.status_code == HTTP_200_OK


def test_user_can_update_ethic(client, ethic):
    response = update_ethic(client, ethic)

    assert response.status_code == HTTP_200_OK


def test_anonymous_user_cannot_update_ethic(anonymous_client, ethic):
    response = update_ethic(anonymous_client, ethic)

    assert response.status_code == HTTP_403_FORBIDDEN


def update_ethic(client, ethic):
    return client.patch(
        reverse('ethic-detail', kwargs=dict(pk=ethic.pk)),
        data=dict(code='foobar', project=ethic.project.pk)
    )



import pytest
import json

from rest_framework.reverse import reverse
from rest_framework.status import HTTP_201_CREATED, HTTP_403_FORBIDDEN

pytestmark = pytest.mark.django_db


def test_admin_user_can_create_ethic(admin_client, ethic_board, project):
    response = create_ethic(admin_client, ethic_board, project)

    assert response.status_code == HTTP_201_CREATED


def test_user_without_permission_can_create_ethic(django_db_reset_sequences, client, ethic_board,
                                                  project):
    response = create_ethic(client, project, ethic_board)

    assert response.status_code == HTTP_201_CREATED, json.dumps(response.json(),
                                                                indent=4,
                                                                sort_keys=True)


def test_anonymous_user_cannot_create_ethic(anonymous_client, project, ethic_board):
    response = create_ethic(anonymous_client, project, ethic_board)

    assert response.status_code == HTTP_403_FORBIDDEN


def create_ethic(client, ethic_board, project):
    data = dict(
        code='123/12-ek',
        ethic_board=ethic_board.id,
        project=project.id,
    )
    return client.post(reverse('ethic-list'), data=data)



from rest_framework.reverse import reverse
from rest_framework.status import HTTP_200_OK, HTTP_403_FORBIDDEN, HTTP_405_METHOD_NOT_ALLOWED


def test_retrieve_is_not_allowed(client, ethic_file):
    response = retrieve_ethic_file(client, ethic_file)

    assert response.status_code == HTTP_405_METHOD_NOT_ALLOWED


def test_admin_user_can_list_ethic_files(admin_client):
    response = list_ethic_files(admin_client)

    assert response.status_code == HTTP_200_OK


def test_user_without_permission_can_list_ethic_file(client):
    response = list_ethic_files(client)

    assert response.status_code == HTTP_200_OK


def test_anonymous_user_cannot_list_ethic_files(anonymous_client):
    response = list_ethic_files(anonymous_client)

    assert response.status_code == HTTP_403_FORBIDDEN


def list_ethic_files(client):
    return client.get(reverse('ethicfile-list'))


def retrieve_ethic_file(client, ethic_file):
    return client.get(reverse('ethicfile-detail', kwargs=dict(pk=ethic_file.id)))



from django_filters import rest_framework as filters

from huscy.project_ethics import models


class EthicFilter(filters.FilterSet):
    class Meta:
        model = models.Ethic
        fields = (
            'project',
        )



import pytest

from rest_framework.reverse import reverse
from rest_framework.status import HTTP_201_CREATED, HTTP_403_FORBIDDEN

from huscy.project_ethics.models import EthicFile

pytestmark = pytest.mark.django_db


def test_admin_user_can_create_ethic_files(admin_client, ethic, tmp_file):
    response = upload_ethic_file(admin_client, ethic, tmp_file)

    assert response.status_code == HTTP_201_CREATED


def test_user_without_permission_can_create_ethic_files(client, ethic, tmp_file):
    response = upload_ethic_file(client, ethic, tmp_file)

    assert response.status_code == HTTP_201_CREATED


def test_anonymous_user_cannot_create_ethic_files(anonymous_client, ethic, tmp_file):
    response = upload_ethic_file(anonymous_client, ethic, tmp_file)

    assert response.status_code == HTTP_403_FORBIDDEN


def upload_ethic_file(client, ethic, tmp_file):
    with open(tmp_file, 'r') as f:
        data = dict(
            ethic=ethic.id,
            filehandle=f,
            filetype=EthicFile.TYPE.get_value('proposal'),
        )
        return client.post(reverse('ethicfile-list'), data=data)



from rest_framework.reverse import reverse
from rest_framework.status import HTTP_200_OK, HTTP_403_FORBIDDEN


def test_admin_user_can_list_ethic_boards(admin_client):
    response = list_ethic_boards(admin_client)

    assert response.status_code == HTTP_200_OK


def test_user_can_list_ethic_boards(client):
    response = list_ethic_boards(client)

    assert response.status_code == HTTP_200_OK


def test_anonymous_user_cannot_list_ethic_boards(anonymous_client):
    response = list_ethic_boards(anonymous_client)

    assert response.status_code == HTTP_403_FORBIDDEN


def list_ethic_boards(client):
    return client.get(reverse('ethicboard-list'))



from huscy.project_ethics.filters import EthicFilter
from huscy.project_ethics.models import Ethic, EthicBoard, EthicFile


def create_ethic_file(ethic, filehandle, filetype, creator):
    filename = filehandle.name.split('/')[-1]

    return EthicFile.objects.create(
        ethic=ethic,
        filehandle=filehandle,
        filename=filename,
        filetype=filetype,
        uploaded_by=creator.get_full_name(),
    )


def get_ethic_files():
    return EthicFile.objects.all()


def get_or_create_ethics(project=None):
    queryset = Ethic.objects.order_by('pk')
    if project is not None:
        filters = dict(project=project.pk)
        filtered_queryset = EthicFilter(filters, queryset).qs
        if not filtered_queryset.exists():
            Ethic.objects.create(project=project)
        return filtered_queryset
    return queryset


def get_ethic_boards():
    return EthicBoard.objects.all()



from enum import Enum

from django.db import models

from huscy.projects.models import Project


class EthicBoard(models.Model):
    name = models.CharField(max_length=255, unique=True)


class Ethic(models.Model):
    code = models.CharField(max_length=255, blank=True, default='')
    ethic_board = models.ForeignKey(EthicBoard, on_delete=models.PROTECT, null=True, blank=True)
    project = models.ForeignKey(Project, on_delete=models.CASCADE)


class EthicFile(models.Model):
    class TYPE(Enum):
        proposal = (0, 'Proposal')
        votum = (1, 'Votum')
        amendment = (2, 'Amendment')

        @classmethod
        def get_value(cls, member):
            return cls[member].value[0]

    def get_upload_path(self, filename):
        return f'ethics/{filename}'

    ethic = models.ForeignKey(Ethic, on_delete=models.CASCADE)

    filetype = models.PositiveSmallIntegerField(choices=[x.value for x in TYPE])

    filehandle = models.FileField(upload_to=get_upload_path)
    filename = models.CharField(max_length=255)

    uploaded_at = models.DateTimeField(auto_now_add=True, editable=False)
    uploaded_by = models.CharField(max_length=126, editable=False)



from model_bakery import baker

from rest_framework.reverse import reverse

from huscy.project_ethics.serializer import EthicSerializer


def test_projects_with_existing_ethics(client, project):
    ethics = baker.make('project_ethics.Ethic', project=project, _quantity=1)

    response = retrieve_project(client, project)

    result = response.json()
    assert result['ethics'] == EthicSerializer(ethics, many=True).data


def test_projects_with_not_existing_ethics(client, project):
    response = retrieve_project(client, project)

    result = response.json()
    assert len(result['ethics']) == 1
    assert result['ethics'][0]['project'] == project.id


def retrieve_project(client, project):
    return client.get(reverse('project-detail', kwargs=dict(pk=project.pk)))



import pytest

from rest_framework.reverse import reverse
from rest_framework.status import HTTP_204_NO_CONTENT, HTTP_403_FORBIDDEN

pytestmark = pytest.mark.django_db


def test_admin_user_can_delete_ethic(admin_client, ethic):
    response = delete_ethic(admin_client, ethic)

    assert response.status_code == HTTP_204_NO_CONTENT


def test_user_can_delete_ethic(client, ethic):
    response = delete_ethic(client, ethic)

    assert response.status_code == HTTP_204_NO_CONTENT


def test_anonymous_user_cannot_delete_ethic(anonymous_client, ethic):
    response = delete_ethic(anonymous_client, ethic)

    assert response.status_code == HTTP_403_FORBIDDEN


def delete_ethic(client, ethic):
    return client.delete(reverse('ethic-detail', kwargs=dict(pk=ethic.pk)))



from rest_framework.reverse import reverse
from rest_framework.status import HTTP_405_METHOD_NOT_ALLOWED


def test_update_not_allowed(client, ethic_file):
    response = client.put(reverse('ethicfile-detail', kwargs=dict(pk=ethic_file.pk)), data={})

    assert response.status_code == HTTP_405_METHOD_NOT_ALLOWED



from django.contrib.auth.models import Permission

from rest_framework.reverse import reverse
from rest_framework.status import HTTP_201_CREATED, HTTP_403_FORBIDDEN


def test_admin_user_can_create_ethic_board(admin_client):
    response = create_ethic_board(admin_client)

    assert response.status_code == HTTP_201_CREATED


def test_user_with_permission_can_create_ethic_board(client, user):
    create_permission = Permission.objects.get(codename='add_ethicboard')
    user.user_permissions.add(create_permission)

    response = create_ethic_board(client)

    assert response.status_code == HTTP_201_CREATED


def test_user_without_permission_cannot_create_ethic_board(client):
    response = create_ethic_board(client)

    assert response.status_code == HTTP_403_FORBIDDEN


def test_anonymous_user_cannot_create_ethic_board(anonymous_client):
    response = create_ethic_board(anonymous_client)

    assert response.status_code == HTTP_403_FORBIDDEN


def create_ethic_board(client):
    return client.post(reverse('ethicboard-list'), data=dict(name='foo Bar'))



from rest_framework.reverse import reverse
from rest_framework.status import HTTP_405_METHOD_NOT_ALLOWED


def test_retrieve_ethics_is_not_provided(client, ethic):
    response = client.get(reverse('ethic-detail', kwargs=dict(pk=ethic.pk)))

    assert response.status_code == HTTP_405_METHOD_NOT_ALLOWED


def test_list_ethics_is_not_provided(client):
    response = client.get(reverse('ethic-list'))

    assert response.status_code == HTTP_405_METHOD_NOT_ALLOWED



import pytest

from rest_framework.reverse import reverse
from rest_framework.status import HTTP_200_OK, HTTP_403_FORBIDDEN


def test_admin_user_can_retrieve_project(admin_client, project):
    response = retrieve_project(admin_client, project)

    assert response.status_code == HTTP_200_OK


def test_user_without_permission_can_retrieve_project(client, project):
    response = retrieve_project(client, project)

    assert response.status_code == HTTP_200_OK


@pytest.mark.django_db
def test_anonymous_user_cannot_retrieve_project(anonymous_client, project):
    response = retrieve_project(anonymous_client, project)

    assert response.status_code == HTTP_403_FORBIDDEN


def retrieve_project(client, project):
    return client.get(reverse('project-detail', kwargs=dict(pk=project.pk)))
